//defines for unused parts of allegro
#define alleg_mouse_unused
//#define alleg_timer_unused
//#define alleg_keyboard_unused
#define alleg_joystick_unused
//#define alleg_gfx_driver_unused
//#define alleg_palette_unused
//#define alleg_graphics_unused
//#define alleg_vidmem_unused
#define alleg_flic_unused
#define alleg_sound_unused
//#define alleg_file_unused
#define alleg_datafile_unused
#define alleg_math_unused
#define alleg_gui_unused

#include <allegro.h>
#include <jrnd.h>

DECLARE_COLOR_DEPTH_LIST(
 COLOR_DEPTH_8
)

int scrx, scry;
int density;
int rate;
float maxspeed;
int lcol, hcol;

volatile unsigned char nextf = 0;
void frame(...) {
  nextf = 1;
}
END_OF_FUNCTION(frame);

class SnowFlake {
private:
  float x, y;
  float xinc, yinc;
  int color;
  bool CheckBound() {return (x >= scrx || y >= scry);}
public:
  SnowFlake() {x=0.0; y=0.0; xinc=1.0; yinc=1.0; color=63;}
  ~SnowFlake() {}
  void MakeNew();
  void Update();
  void Gust(int move);
};

int main() {
  allegro_init();

  //Open Config file and read variables
  set_config_file("snow.cfg");
  scrx = get_config_int("screen", "xres", 320);
  scry = get_config_int("screen", "yres", 200);
  density = get_config_int("simulation", "density", 1000);
  rate = get_config_int("simulation", "rate", 20);
  maxspeed = get_config_float("simulation", "speed", 3.0);
  lcol = get_config_int("simulation", "lowcolor", 15);
  hcol = get_config_int("simulation", "highcolor", 50);

  randomize(); //set up RNG
  install_timer();
  LOCK_FUNCTION(frame);
  LOCK_VARIABLE(nextf);
  install_keyboard();
  set_color_depth(8);
  set_gfx_mode(GFX_AUTODETECT, scrx, scry, 0, 0);

  int numbsnow = 0;
  SnowFlake *flakes = new SnowFlake[density]; //grab memory
  SnowFlake *snowflakes = flakes; //temp var for cycling the array
  for (int c=0; c<density; c++)   //init array with new flakes
    (snowflakes++)->MakeNew();

  vsync();
  for (int c=0; c<63; c++) { //init colors
    RGB col;
    col.r = col.g = col.b = c;
    _set_color(c, &col);
  }

  install_int(frame, 20);
  while(!keypressed()) {
    snowflakes = flakes;
    if((numbsnow < density) && (getrnd(rate) == 0))
      numbsnow++; //incr numbsnow to update more flakes

    for (int c=0; c<numbsnow; c++)
      (snowflakes++)->Update();

    while (!nextf) {}
    nextf = 0;
  }

  delete[] flakes; //deallocate memory
  allegro_exit();
  return 0;
}

void SnowFlake::MakeNew() {
  x = y = 0;
  if (getrnd(2))
    x = getrnd(scrx);
  else
    y = getrnd(scry);
  xinc = getrndf(maxspeed) + 1.0;
  yinc = getrndf(maxspeed) + 1.0;
  color= getrnd(hcol, lcol);
};

void SnowFlake::Update() {
  _putpixel(screen, int(x), int(y), 0);
  x += xinc;
  y += yinc;
  if (CheckBound())
    MakeNew();
  _putpixel(screen, int(x), int(y), color);
}

